from moviepy.editor import VideoFileClip
from os import path as os_path
from loguru import logger


def mp4(src: str, dst: str) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == 'mp3':
        mp4_to_mp3(src, dst)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def mp4_to_mp3(src: str, dst: str) -> None:
    video = VideoFileClip(src)
    audio = video.audio
    audio.write_audiofile(dst)
    return



from fastapi import APIRouter
from fastapi.templating import Jinja2Templates
from loguru import logger
from starlette.requests import Request
from starlette.responses import HTMLResponse

from ..apis import example
from ..model import Example

router = APIRouter(
    prefix="",
    tags=["home"],
)

templates = Jinja2Templates(directory="templates")


@router.get("/", response_class=HTMLResponse)
async def home(request: Request):
    m: Example = Example(msg="Hello World")
    d = example(e=m)
    logger.debug(f"API call returned data: {d}")
    return templates.TemplateResponse("index.html", {"request": request, "data": d})



from os import getcwd
from os import path as os_path

from click import group
from click import option
from loguru import logger

from .m4a import m4a
from .mov import mov
from .mp3 import mp3
from .mp4 import mp4
from .png import png
from .png import to_png
from .webm import webm


@group(name='conv', help="Convert a file")
def conv():
    pass


@conv.command(help="List all available conversions")
def list():
    logger.debug("conv list")

    print("Available conversions:")
    print("  video")
    print("    mp4,webm,mov -> mp3")
    print("  audio")
    print("    m4a -> wav, mp3")
    print("    mp3 -> wav, m4a")
    print("  image")
    print("    jpg, jpeg, webp -> png")
    print("    jpg, jpeg, webp, png -> mp4")
    return


@conv.command(help="Convert a video to another format")
@option('-p', '--path', type=str, required=True, prompt=True,
        help="Path to the video file (e.g. /path/to/file)")
@option('-f', '--format', type=str, required=True, prompt=True,
        help="Format to convert to (e.g. mp3)")
def video(path: str, format: str):
    logger.debug("conv video")

    if not os_path.isabs(path):
        path = os_path.join(getcwd(), path)

    if not os_path.exists(path):
        print(f"Video {path} does not exist")
        return

    splits = path.split(".")

    src_format = splits[-1]
    src_filename = ".".join(splits[:-1])
    src_path = path

    dst_format = format
    dst_path = f"{src_filename}.{dst_format}"

    logger.debug(f"Converting {src_path} to {dst_path}")

    if src_format == 'mp4':
        mp4(src_path, dst_path)
    elif src_format == 'webm':
        webm(src_path, dst_path)
    elif src_format == 'mov':
        mov(src_path, dst_path)
    else:
        logger.error(f"Unsupported file {src_format}")
    return


@conv.command(help="Convert an audio to another format")
@option('-p', '--path', type=str, required=True, prompt=True,
        help="Path to the audio file (e.g. /path/to/file)")
@option('-f', '--format', type=str, required=True, prompt=True,
        help="Format to convert to (e.g. wav)")
def audio(path: str, format: str):
    logger.debug("conv audio")

    if not os_path.isabs(path):
        path = os_path.join(getcwd(), path)

    if not os_path.exists(path):
        print(f"Audio {path} does not exist")
        return

    splits = path.split(".")

    src_format = splits[-1]
    src_filename = ".".join(splits[:-1])
    src_path = path

    dst_format = format
    dst_path = f"{src_filename}.{dst_format}"

    logger.debug(f"Converting {src_path} to {dst_path}")

    if src_format == 'm4a':
        m4a(src_path, dst_path)
    elif src_format == 'mp3':
        mp3(src_path, dst_path)
    else:
        logger.error(f"Unsupported file {src_format}")
    return


@conv.command(help="Convert an image into another format")
@option('-p', '--path', type=str, required=True, prompt=True,
        help="Path to the image file (e.g. /path/to/file)")
@option('-f', '--format', type=str, required=True, prompt=True,
        help="Format to convert to (e.g. jpg, mp4)")
@option('-d', '--duration', type=int, default=0,
        help="Duration of output file in seconds (e.g. 30)")
def image(path: str, format: str, duration: int):
    logger.debug("conv image")

    if not os_path.isabs(path):
        path = os_path.join(getcwd(), path)

    if not os_path.exists(path):
        print(f"Image {path} does not exist")
        return

    splits = path.split(".")

    src_format = splits[-1]
    src_filename = ".".join(splits[:-1])
    src_path = path

    dst_format = format
    dst_path = f"{src_filename}.{dst_format}"

    if src_format != 'png':
        src_path = to_png(src_path)
    if dst_format == 'png':
        return

    logger.debug(f"Converting {src_path} to {dst_path}")

    png(src_path, dst_path, duration)
    return



from loguru import logger
from moviepy.editor import AudioFileClip
from moviepy.editor import VideoFileClip


def mov(src: str, dst: str) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == 'mp3':
        try:
            mov_video_to_mp3(src, dst)
        except Exception as e:
            logger.debug(f"Failed to convert {src} to {dst} as video, retrying as audio")
            mov_audio_to_mp3(src, dst)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def mov_video_to_mp3(src: str, dst: str) -> None:
    video = VideoFileClip(src)
    audio = video.audio
    audio.write_audiofile(dst)
    return


def mov_audio_to_mp3(src: str, dst: str) -> None:
    audio = AudioFileClip(src)
    audio.write_audiofile(dst)
    return



from click import group
from loguru import logger


@group(name='sync', help="Sync files")
def sync():
    pass


@sync.command()
def zshrc():
    logger.debug("sync .zshrc")


@sync.command()
def vimrc():
    logger.debug("sync .vimrc")


@sync.command()
def gitconfig():
    logger.debug("sync .gitconfig")



from os import path as os_path

from pydantic import BaseModel

PD_CONFIG_FILE = os_path.expanduser('~/.pdconfig.json')


class Project(BaseModel):
    """
    Each project is uniquely identified by its full path on disk
    """

    type: str  # Type of project (e.g. react, fastapi)
    name: str  # Name of the project (e.g. project)
    path: str  # Full path on disk (e.g. /path/to/project)


class Instance(BaseModel):
    """
    Each instance is uniquely identified by its ID
    """

    id: str  # Instance ID
    ip: str  # Instance IP (e.g. Elastic IP)



from importlib import resources
from os import path as os_path
from shutil import copytree
from shutil import ignore_patterns

from click import group
from click import option
from loguru import logger

from ..config.config import manage_projects


@group(name='init', help="Initialize a new project")
def init():
    pass


@init.command(help="Start a new FastAPI App")
@option('-n', '--name', type=str, required=True, prompt=True,
        help="Project name e.g. project (or /path/to/project)")
@option('-f', '--force', type=bool, is_flag=True, default=False,
        help="Force overwrite of existing files")
def fastapi(name: str, force: bool):
    logger.debug("init fastapi")

    path, name = (name.rsplit('/', 1) if '/' in name else (".", name))
    full_path = os_path.join(path, name)

    if not force:
        if os_path.exists(full_path):
            logger.error(f"Path {full_path} already exists")
            return

    logger.debug(f"path: {path}")
    logger.debug(f"name: {name}")
    logger.debug(f"full_path: {full_path}")

    ignore_dirs = ignore_patterns('__pycache__', 'venv', 'node_modules')
    with resources.path('pd.init.templates', 'pd-fastapi') as template_path:
        logger.debug(f"Copying {template_path} to {full_path}")
        copytree(template_path, full_path, dirs_exist_ok=force, ignore=ignore_dirs)

    manage_projects(add=f"fastapi,{full_path}")

    print(f'Created project {name} at {full_path}. Please run the following commands:')
    print("cd", full_path)
    print("pip install -r requirements.txt")
    print("npm install")
    print("python3 -m uvicorn app.main:app --reload")
    print("Open http://localhost:8000")
    return


@init.command(help="Start a new React App")
@option('-n', '--name', type=str, required=True, prompt=True,
        help="Project name e.g. project (or /path/to/project)")
@option('-f', '--force', type=bool, is_flag=True, default=False,
        help="Force overwrite of existing files")
def react(name: str, force: bool):
    logger.debug("init react")

    path, name = (name.rsplit('/', 1) if '/' in name else (".", name))
    full_path = os_path.join(path, name)

    if not force:
        if os_path.exists(full_path):
            logger.error(f"Path {full_path} already exists")
            return

    logger.debug(f"path: {path}")
    logger.debug(f"name: {name}")
    logger.debug(f"full_path: {full_path}")

    ignore_dirs = ignore_patterns('node_modules')
    with resources.path('pd.init.templates', 'pd-react') as template_path:
        logger.debug(f"Copying {template_path} to {full_path}")
        copytree(template_path, full_path, dirs_exist_ok=force, ignore=ignore_dirs)

    manage_projects(add=f"react,{full_path}")

    print(f'Created project {name} at {full_path}. Please run the following commands:')
    print(full_path)
    print("cd", full_path)
    print("npm install")
    print("npm start dev")
    print("Open http://localhost:3000")
    return


@init.command(help="Start a new Electron App")
@option('-n', '--name', type=str, required=True, prompt=True,
        help="Project name e.g. project (or /path/to/project)")
@option('-f', '--force', type=bool, is_flag=True, default=False,
        help="Force overwrite of existing files")
def electron(name: str, force: bool):
    logger.debug("init electron")

    path, name = (name.rsplit('/', 1) if '/' in name else (".", name))
    full_path = os_path.join(path, name)

    if not force:
        if os_path.exists(full_path):
            logger.error(f"Path {full_path} already exists")
            return

    logger.debug(f"path: {path}")
    logger.debug(f"name: {name}")
    logger.debug(f"full_path: {full_path}")

    ignore_dirs = ignore_patterns('node_modules')
    with resources.path('pd.init.templates', 'pd-electron') as template_path:
        logger.debug(f"Copying {template_path} to {full_path}")
        copytree(template_path, full_path, dirs_exist_ok=force, ignore=ignore_dirs)

    manage_projects(add=f"electron,{full_path}")

    print(f'Created project {name} at {full_path}. Please run the following commands:')
    print(full_path)
    print("cd", full_path)
    print("npm install")
    print("npm start")
    return



from sys import stderr
from sys import stdout

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi.staticfiles import StaticFiles
from loguru import logger

from .routers import home

logger.remove(0)
logger.add(stdout, level="DEBUG")
logger.add(stderr, level="ERROR")
logger.add("logs/app.log", level="DEBUG", rotation="1 day", retention="1 week")

app = FastAPI()
app.mount("/static", StaticFiles(directory="static"), name="static")

app.include_router(home.router)

origins = [
    # Replace this with your domain
    "*",
]

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)
